################################################################################
# File    : export.tcl
# Content : a script to get mangled names from .obj files with a specific
#           pattern (typically a namespace identifier)
# History : 27-Nov-2001 - [rb] creation
#           29-Jan-2002 - [rb] updated to use dumpbin instead of nm to dump
#                         object file content
################################################################################

#   Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# export --
#
# Build a list of symbols matching a pattern (usualy a namespace) that are in
# Win32 object files.
#
# Parameters
# arg is a list where
#     - the first element is the desired output file name
#     - the second element is the pattern
#     - the other elements are object filenames

proc export {arg} {

	puts "export.tcl - generation of exported symbols from object files."

	set outname [lindex $arg 0]
	set pattern [lindex $arg 1]

	set objects ""
	set data ""
	set code ""

	foreach obj [lrange $arg 2 end] {
		append objects "$obj "

# nm version -------------------------------------------------------------------
#		set lines [split [exec nm $obj] "\n"]
# dumpbin version --------------------------------------------------------------
		set lines [split [exec dumpbin /SYMBOLS $obj] "\n"]
# ------------------------------------------------------------------------------

		foreach line $lines {

# nm version -------------------------------------------------------------------
#			set symbol [lindex [split $line " "] end]
# dumpbin version --------------------------------------------------------------
			set symbol [lindex [split [string range $line [expr [string first "|" $line] + 2] end] " "] 0]
# ------------------------------------------------------------------------------

			if [regexp -- $pattern $symbol] {
				# remove scalar destructors ???
				if {([string first "??_G" $symbol] == -1) &&
				    ([string first "??_E" $symbol] == -1)} {
					# data
					if {[string index $symbol end] == "A"} {
						append data "$symbol\n"
					} else {
						append code "$symbol\n"
					}
				}
			}
		}
	}

	# header
	set header "EXPORTS\n"

	# writing output
	set out [open $outname w]
	puts $out $header
	puts $out $data
	puts $out $code
	close $out

	puts "$outname - generated from $objects"

	return -code ok
}

if [catch {export $argv}] {
	puts stderr "Usage : tclsh export.tcl <output file> <pattern> file..."
	exit 1
}

