////////////////////////////////////////////////////////////////////////////////
// File    : player.cpp
// Content : a svg viewer
// History : 11-Jun-2003 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <interactions.h>

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>
#include <hsm/system/plugin/PlugIn.h>

#include <svgl/Parser.hpp>

#include <w3c/svg/SVGSVGElement.hpp>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "player - svg viewer using svgl." << endl << flush;
	atexit(quitHandler);
	
	if(argc != 2) {
		cerr << "usage : player file.svg" << endl;
		return EXIT_FAILURE;
	}
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("player");
	hsm::EventHandler *pointer = hsm::repository::getEventHandler("Pointer");
	window->addEventHandler(pointer);

	// creating instruments
	Clicker::Hsm *clicker = new Clicker::Hsm(Clicker::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	Zoomer::Hsm *zoomer = new Zoomer::Hsm(Zoomer::Data(window, *pointer/"buttons"/hsm::ButtonPad::R, *pointer/"position"));
	
	// getting the svg document
	svgl::Parser parser;
	svg::SVGDocument *document = 0;
	document = parser.parse(argv[1]);
	if(document == 0) {
		return EXIT_FAILURE;
	}
	
	window->setDocument(document);
	window->initBehaviours();


	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	


	// clean-up
	delete clicker;
	delete enterLeaver;
	delete translater;
	delete zoomer;
	
	delete window;
	

	return EXIT_SUCCESS;
}