////////////////////////////////////////////////////////////////////////////////
// File    : ButtonPad.h
// Content : ButtonPad component
// History : 26-Nov-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// - add assignement operators


#if !defined(HSM_BUTTONPAD_H)
#define HSM_BUTTONPAD_H


/* includes *******************************************************************/

#include <hsm/core/Composite.h>
#include <hsm/component/Button.h>

#include <vector>
#include <ostream>


namespace hsm {

/* ButtonPad class definition *************************************************/

	class ButtonPad : public impl::Component< Composite > {
	public :
		static const ButtonPad *cast;
		
		enum ButtonId {
			L = 0, R, M,
			SHIFT = 0, ALT, CTRL,
			MAX = 255
		};

	protected :
		unsigned int dimension;
		std::vector< Button * > buttons;

	public :
		         ButtonPad(const Id &id, unsigned int d);
		explicit ButtonPad(unsigned int d);
		         ButtonPad(const ButtonPad &b);
		virtual ~ButtonPad();

	private :
		void construct();

	protected :
		virtual void process(const Event &event);

	public :
		ButtonPad &operator=(unsigned int bits);

		inline Button &operator/(ButtonPad::ButtonId i) const { return *(buttons[i]); }
		inline Button &operator/(char i) const { return *(buttons[int(i)]); }
		friend Button &operator/(const Component &buttonPad, ButtonPad::ButtonId i);
		friend Button &operator/(const Component &buttonPad, char i);
		friend std::ostream &operator<<(std::ostream &s, const ButtonPad &buttonPad);
	};


} // namespace hsm


#endif // !defined(HSM_BUTTONPAD_H)