////////////////////////////////////////////////////////////////////////////////
// File    : Point.h
// Content : Point component
// History : 26-Nov-2001 - [rb] creation
//           28-Feb-2002 - [rb] added assignement operator for Point class
//           31-Jan-2003 - [rb] removed automatic dispatching from Values to
//                              Point and threshold for Point
//           17-Feb-2004 - [rb] added a way to set multiples dimensions
//                              simultaneously, so removed the need of explicit
//                              dispatching
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo(-) / Done(+) ///////////////////////////////////////////////////////////
// + don't make dispatch public for Point -> need a way to set the multiple
//   dimensions simultaneously


#if !defined(HSM_POINT_H)
#define HSM_POINT_H


/* includes *******************************************************************/

#include <hsm/core/Composite.h>
#include <hsm/component/Value.h>

#include <vector>
#include <ostream>


namespace hsm {

/* Point class definition *****************************************************/

	class Point : public impl::Component< Composite > {
	public :
		static const Point *cast;

		enum ValueId {
			X = 0, Y, Z,
			W = 0,
			T = 0,
			Az = 0,	Al,
			P = 0,
			R = 0, G, B, A,
			MAX = 255
		};

	protected :
		unsigned int dimension;
		unsigned int current;
		std::vector< Value * > values;
		
		float threshold2;
		float diff2;

	public :
		         Point(const Id &id, unsigned int d, float th = 1.);
		explicit Point(int d, float th = 1.);
		virtual ~Point();

	private :
		void construct();
		
	protected :
		virtual void process(const Event &event);

	public :
		Point &operator=(float f);
		Point &operator,(float f);

		Point &operator=(const Point &p);
		Point &operator+=(const Point &p);
		Point &operator-=(const Point &p);

		Point &operator*=(float f);
		Point &operator/=(float f);
		
		float operator*(const Point &point) const;
		
		friend float abs2(const Point &p);
		friend float abs(const Point &p);

		bool operator<=(const Point &p) const;
		bool operator>=(const Point &p) const;

		inline Value &operator/(Point::ValueId i) const { return *(values[i]); }
		friend Value &operator/(const Component &point, Point::ValueId i);
		friend std::ostream &operator<<(std::ostream &s, const Point &point);
	};
	
	
} // namespace hsm


#endif // !defined(HSM_POINT_H)
