////////////////////////////////////////////////////////////////////////////////
// File    : Svgl.cpp
// Content : svgl helper functions
// History : 30-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Svgl.h>

#include <w3c/svg/SVGTransforms.hpp>
#include <w3c/svg/SVGTransformable.hpp>


namespace hsm {

	namespace svgl {
	

/* implementation *************************************************************/

		void getGlobalTransform(svg::Element *e, float &sx, float &sy, float &tx, float &ty, bool skipFirst /* = true */) {
			if(e == 0) {
				sx = 1.; sy = 1.;
				tx = 0.; ty = 0.;
				return;
			}
			
			getGlobalTransform(dynamic_cast< svg::Element * >(e->getParentNode()), sx, sy, tx, ty, false);

			if(skipFirst) {
				return;
			}
			
			svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(e);
			if(transformable == 0) {
				return;
			}

			const svg::SVGTransformList &transforms = transformable->getTransform();
			unsigned int noi = transforms.numberOfItems();
			for(unsigned int i = 0; i < noi; ++i) {
				svg::SVGTransform *transform = transforms.getItem(i);
				svg::SVGTranslateTransform *trans = dynamic_cast< svg::SVGTranslateTransform * >(transform);
				if(trans != 0) {
					tx += trans->tx * sx;
					ty += trans->ty * sy;
				} else {
					svg::SVGScaleTransform *scale = dynamic_cast< svg::SVGScaleTransform * >(transform);
					if(scale != 0) {
						sx *= scale->sx;
						sy *= scale->sy;
					}
				}
			}
			
		}


		std::string getAttributeCC(svg::Element *e, const char *attribute) {
			unicode::String *us = e->getAttribute(unicode::String::createString(attribute));
			if(us != 0) {
				return us->transcode();
			} else {
				return "0";
			}
		}
		
		void setAttributeCC(svg::Element *e, const char *attribute, const char *value) {
			e->setAttribute(unicode::String::createString(attribute), unicode::String::createString(value));
		}
		
		
	} // namespace svgl
	
} // namespace hsm
