////////////////////////////////////////////////////////////////////////////////
// File    : File.cpp
// Content : file relative platform dependent declaration
// History : 09-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <File.h>

#if !defined(WIN32)
#  include <sys/ioctl.h> // ioctl
#else
#  include <sstream>
#endif


namespace hsm {

	namespace file {

/* functions ******************************************************************/

		int bytesToRead(Descriptor id) {
			int n;
#if !defined(WIN32) ////////////////////////////////////////////////////////////
			ioctl(id, FIONREAD, &n);
#else //////////////////////////////////////////////////////////////////////////
			PeekNamedPipe(id, 0, 0, 0, (DWORD *)(&n), 0);
#endif /////////////////////////////////////////////////////////////////////////
			return n;
		}


#if defined(WIN32) /////////////////////////////////////////////////////////////

		int close(Descriptor fildes) {
			return CloseHandle(fildes);
		}

		static unsigned int pn;

		int pipe(Descriptor *fildes) {
			std::stringstream name;
			name << "\\\\.\\pipe\\HSM" << (pn++);
			fildes[0] = CreateNamedPipe(name.str().c_str(),
			                            PIPE_ACCESS_INBOUND | FILE_FLAG_OVERLAPPED,
			                            0, 1, 4096, 4096, 0, 0);
			fildes[1] = CreateFile(name.str().c_str(), GENERIC_WRITE, 0, 0,
			                       OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
			return 0;
		}

		int read(Descriptor fildes, void *buf, int nbyte) {
			int n, n0;
			ReadFile(fildes, buf, nbyte, (DWORD *)(&n), 0);
			// performing an empty read returns to unsignaled state if the pipe is empty
			ReadFile(fildes, 0, 0, (DWORD *)(&n0), 0);
			return n;
		}

		int write(Descriptor fildes, void *buf, int nbyte) {
			int n;
			WriteFile(fildes, buf, nbyte, (DWORD *)(&n), 0);
			return n;
		}

#endif /////////////////////////////////////////////////////////////////////////


	} // namespace file

} // namespace hsm
