////////////////////////////////////////////////////////////////////////////////
// File    : test-Functor.cpp
// Content : a test for hsm Functor
// History : 17-Jan-2002 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cassert>
#include <cstdlib>
#include <iostream>
#include <string>
using namespace std;

#include <hsm/util/Functor.h>


/* globals ********************************************************************/


class Foo {
protected :
	string name;
public :
	Foo(string n) : name(n) {}
	virtual ~Foo() {}
	
	static void staticBar() { cout << "Foo::staticBar()" << endl; }
	virtual void bar() { cout << name << "'s Foo::bar()" << endl; }
	virtual void bar2() { cout << "coucou"; }
};

class DFoo : public Foo {
public :
	DFoo(string n) : Foo(n) {}
	
	virtual void bar() { cout << name << "'s DFoo::bar()" << endl; }
	virtual void bar2() { cout << "couu"; }
};

void voidFunction() { cout << "voidFunction()" << endl; }
void pVoidFunction(void *data) {
  if(data == 0) {
    cout << "pVoidFunction(0)" << endl;
  } else {
    cout << "pVoidFunction('" << *reinterpret_cast< char * >(data) << "')" << endl;
  }
}

/* handle quit ****************************************************************/

void quitHandler() {
	cout << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-Functor - a program to test the hsm::Functor class." << endl;
	atexit(quitHandler);

	char a ='a';
	
	hsm::Functor *f = hsm::Functor::create(voidFunction);
	(*f)();
	delete f;

	f = hsm::Functor::create(pVoidFunction);
	(*f)();
	(*f)(reinterpret_cast< void * >(&a));
	delete f;

	f = hsm::Functor::create(Foo::staticBar);
	(*f)();
	delete f;

	Foo *foo = new Foo("foo");
	f = hsm::Functor::create(foo, &Foo::bar);
	(*f)();
	delete f;
	delete foo;
	
	foo = new DFoo("dfoo");
	f = hsm::Functor::create(foo, &Foo::bar);
	(*f)();
	delete f;
	delete foo;


	return EXIT_SUCCESS;
}
