////////////////////////////////////////////////////////////////////////////////
// File    : test-GLWindow.cpp
// Content : a test for hsm::GLWindow
// History : 18-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
using namespace std;

#include <hsm/system/window/GLWindow.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/Point.h>
#include <hsm/core/Component.h>


/* globals ********************************************************************/

class GLWindow : public hsm::GLWindow {
public :
	hsm::Point p;
	hsm::EventHandler *pointer;
	hsm::Point &mouse;
	
public :
	GLWindow(const Id &id)
	 : hsm::GLWindow(id), p(2), pointer(hsm::repository::getEventHandler("Pointer")), mouse(*pointer/"position"/hsm::Point::cast) {
		addEventHandler(pointer);
		mouse.addReceiver(this);
	}
	
	~GLWindow() {
		mouse.removeReceiver(this);
	}

protected :
	
	virtual void display() {
		hsm::GLWindow::display();
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glTranslatef(p/hsm::Point::X, p/hsm::Point::Y, 0.0);
		glColor3f(.5, .2, .7);
		glBegin(GL_POLYGON);
			glVertex2f(-5.0, -5.0);
			glVertex2f(5.0, -5.0);
			glVertex2f(5.0, 5.0);
			glVertex2f(-5.0, 5.0);
		glEnd();
	}
	
	virtual void process(const hsm::Event &event) {
		hsm::GLWindow::process(event);
		if((event.sender == &mouse) && (event.message != hsm::Event::DELETED)) {
			screen2window(mouse, p);
		}
	}
};


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-GLWindow - a program to test the hsm::Window class." << endl << endl;
	atexit(quitHandler);
	
	hsm::GLWindow *sglw = new hsm::impl::Component< GLWindow >("Sync GLWindow");
	sglw->setTitle("Sync GLWindow");
	sglw->show();

	hsm::GLWindow *aglw = new hsm::impl::Component< GLWindow, hsm::processing::Async >("Async GLWindow");
	aglw->show();

	hsm::GLWindow *mglw = new hsm::impl::Component< GLWindow, hsm::processing::MonoAsync >("MonoAsync GLWindow");
	mglw->show();

	while(hsm::eventloop::step()) {
		sglw->post();
		aglw->post();
		mglw->post();
	}
	

	delete sglw;
	delete aglw;
	delete mglw;
	
	return EXIT_SUCCESS;
}
