////////////////////////////////////////////////////////////////////////////////
// File    : test-ThreadLoop.cpp
// Author  : Renaud BLANCH
// Content : a test for hsm ThreadLoop
// History : 16-Jan-2002 - creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
using namespace std;

#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/system/thread/Thread.h>
#include <hsm/util/Functor.h>


/* globals ********************************************************************/

void printLoop(void *c) {
	cout << *reinterpret_cast< char * >(c) << flush;
}

void pauseLoop() {
	cout << "Doing something then pausing. "<< endl;
	hsm::ThreadLoop::get()->pause();
}

void stopLoop() {
	cout << "Doing something then stopping. "<< endl;
	hsm::ThreadLoop::get()->stop();
}

class Startor : public hsm::Functor {
public :
	virtual void operator()(void *pData = 0) { cout << "[thread started]"; }
};

class Endor : public hsm::Functor {
public :
	virtual void operator()(void *pData = 0) { cout << "[thread ended]"; }
};



/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-ThreadLoop - a program to test the hsm::ThreadLoop class." << endl;
	atexit(quitHandler);

	char a = 'a';
	char b = 'b';
	
	hsm::Functor *l = 0;

	hsm::ThreadLoop *t1 = 0;
	hsm::ThreadLoop *t2 = 0;


	cout << endl << "--- basic loop that outputs \'a\' every 1/5 sec during 5 sec :" << endl;

	l = hsm::Functor::create(printLoop);
	t1 = new hsm::ThreadLoop(l, 200, reinterpret_cast<void *>(&a), new Startor(), new Endor());

	t1->play();
	hsm::thread::sleep(5000);

	delete t1;

	cout << endl;


	cout << endl << "--- 2 concurent threads outputing \'a\' & \'b\' every 1/2 and 1 sec during 5 sec :" << endl;

	t1 = new hsm::ThreadLoop(l,  500, reinterpret_cast< void * >(&a), new Startor(), new Endor());
	t2 = new hsm::ThreadLoop(l, 1000, reinterpret_cast< void * >(&b), new Startor(), new Endor());

	t1->play(); t2->play();
	hsm::thread::sleep(5000);
	t1->stop(); t2->stop();
	
	delete t1; delete t2;
	delete l;

	cout << endl;

	cout << endl << "--- self pausing thread 5 times :" << endl;

	l = hsm::Functor::create(pauseLoop);
	t1 = new hsm::ThreadLoop(l);

	for(unsigned int i = 0; i < 5; i++) {
		t1->play();
		hsm::thread::sleep(1000);
	}

	delete t1;
	delete l;

	cout << endl << "--- self stopping thread :" << endl;

	l = hsm::Functor::create(stopLoop);
	t1 = new hsm::ThreadLoop(l);

	t1->play();

	hsm::ThreadLoop::waitFor(t1);
	delete t1;
	delete l;


	return EXIT_SUCCESS;
}
